/*
 * Created on Sep 2, 2004
 *
 */
package nl.amis.workshop;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author zeger
 *
 */
public class PersonEntity extends Person implements CRUD
{
    private Log log = LogFactory.getLog(PersonEntity.class);
    
	/* (non-Javadoc)
	 * @see nl.amis.workshop.CRUD#createRecord()
	 */
	public boolean createRecord() 
	{
	    String insertStmtSQL = 
	        "   INSERT INTO person " 
	        + " (initials, first_name, middle_names, last_name)" 
	        + " VALUES (?, ?, ?, ?)";
	    CallableStatement statement = null;
	    boolean success = false;
	    try
	    {
	        // get the sql statement
	        statement = DbConnect.getCallableStatement(insertStmtSQL);
	        
	        // Fill in all the gaps in the sql statement
	        statement.setString(1, this.initials);
	        statement.setString(2, this.firstName);
	        statement.setString(3, this.middleNames);
	        statement.setString(4, this.lastName);
	              
	        int rowsInserted = statement.executeUpdate();
	        // atleast 1 row should be inserted or else 
	        // we've encountered something nasty
	        success = (rowsInserted >= 1);      
	    }
	    catch (SQLException e)
	    {
	        System.out.println(e.getMessage());
	    }
        finally
        {
	        // make sure we close the statement to prevent 
        	// MAX_CURSORS_OPEN
	        try
	        {
	          statement.close();
	        }
	        catch (Exception e)
	        {
	          log.error(e.getMessage());
	        }
	    }
	    return success;
	}

	/* (non-Javadoc)
	 * @see nl.amis.workshop.CRUD#readRecord()
	 */
	public boolean readRecord() 
	{
	    String selectStmtSQL = 
	        "   SELECT initials, first_name, middle_names, last_name "
	        + " FROM person  "
	        + " WHERE id = ?";
	      CallableStatement statement = null;
	      boolean success = false;
	      try
	      {
	        // get the sql statement
	        statement = DbConnect.getCallableStatement(selectStmtSQL);
	        
	        // Fill in all the gaps in the sql statement      
	        // PARAID in where clause
	        statement.setLong(1, this.personId);
	              
	        ResultSet result = statement.executeQuery();
	        // atleast 1 row should be selected or 
	        // else we've encountered something nasty
	        if (result.next())
	        {
	          serializeRecord(result);      
	          result.close();
	          success = true;
	        }
	      }
	      catch (SQLException e)
	      {
	        log.error(e.getMessage());
	      }
	      finally
	      {
	        try
	        {
	          statement.close();
	        }
	        catch (Exception e)
	        {
	          log.error(e.getMessage());
	        }
	      }
	      return success;
	}

	/* (non-Javadoc)
	 * @see nl.amis.workshop.CRUD#updateRecord()
	 */
	public boolean updateRecord() 
    {
	    String updateStmtSQL = 
	        "   UPDATE person " + 
	        " SET initials = ?, first_name = ?, " +
			" middle_names = ?, last_name = ?" + 
	        " WHERE id = ? ";
	    CallableStatement statement = null;
	    boolean success = true;
	    try
	    {
	      // get the sql statement
	      statement = 
              DbConnect.getCallableStatement(updateStmtSQL);
	      
	      statement.setString(1, initials);
          statement.setString(2, firstName);
          statement.setString(3, middleNames);
          statement.setString(4, lastName);
	      
	      statement.executeUpdate();
	    }
	    catch (SQLException e)
	    {
	      success = false;
	      log.error(e.getMessage());
	    }
	    finally
	    {
	      try
	      {
	        statement.close();
	      }
	      catch (Exception e)
	      {
	        System.out.println(e.getMessage());
	      }
	    }
	    return success;
    }
	
	/* (non-Javadoc)
	 * @see nl.amis.workshop.CRUD#deleteRecord()
	 */
	public boolean deleteRecord() 
	{
	    String deleteStmtSQL = 
	        "   DELETE FROM person "    
	        + " WHERE id = ?";  
	      CallableStatement statement = null;
	      boolean success = false;
	      try
	      {
	        // get the sql statement
	        statement = 
                DbConnect.getCallableStatement(deleteStmtSQL);
	        
	        statement.setLong(1, this.personId);
	              
	        int rowsDeleted = statement.executeUpdate();
	        // atleast 1 row should be deleted or else we've encountered something nasty
	        success = (rowsDeleted >= 1);      
	      }
	      catch (SQLException e)
	      {
	        log.error(e.getMessage());
	      }
	      finally
	      {
	        try
	        {
	          statement.close();
	        }
	        catch (Exception e)
	        {
	          System.out.println(e.getMessage());
	        }
	      }
	      return success;
    }	
    
	
    /**
	 * Serializes one record from a ResultSet to this class
	 * @param aRecord is a ResultSet
	 * @throws java.sql.SQLException
	 */
	protected void serializeRecord(ResultSet aRecord)
	  throws SQLException
	{   
	    // serialize row to this date entitity class
	    this.initials = aRecord.getString("initials");
	    this.firstName = aRecord.getString("first_name");
	    this.middleNames = aRecord.getString("middle_names");
	    this.lastName = aRecord.getString("last_name");
	}
	
	public String toString()
	{
		StringBuffer sb = new StringBuffer(getClass().getName() + ": ");
		sb.append("Primary key=" + this.personId + "; ");
		sb.append("Initials='" + this.initials + "'; ");
		sb.append("Firstname='" + this.firstName + "'; ");
		sb.append("Middlenames='" + this.middleNames + "'; ");
		sb.append("Lastname='" + this.lastName + "'; ");
		sb.append(" systemHashcode=" + System.identityHashCode(this));
		return sb.toString();
	}
}
